# SetBookStackMaterial.py
# Script for testing setup of materials in multi-figure BookStack4 stacks

import poser,random

# I put my scripts into a class simply to avoid possible conflict with stuff in the __main__ module.
class cheapskatesTemporaryClass(object):
	
	# List of all the BookStack4 figures linked underneath the selected one
	figlist = []
	
	# Recursive function to get any BookStack4 figures to which this, or any of its parent, BookStack4 figures are parented
	def getAnyParentStackFigures(self,thisfig):
		print ("\ngetAnyParentStackFigures()")
		act = thisfig.ParentActor()
		print ("thisfig.ParentActor() = " + act.Name())
		parentact = act.Parent()
		parentactname = parentact.Name()
		print ("and its parent actor is " + parentact.Name())
		if parentactname[:10] == "TopOfStack":
			if parentact.ItsFigure():
				parentfig = parentact.ItsFigure()
				figname = parentfig.Name()
				if figname[:9] == "BookStack":
					print ("Adding '" + figname +"' to figlist[]")
					self.figlist.append(figname)
					self.getAnyParentStackFigures(parentfig)
		return
	
	# Recursive function to get any BookStack4 figures that are parented to TopOfStack of the main, or of any nested, BookStack4 figure
	def getAnyChildStackFigure(self, thisfig):
		print ("\ngetAnyChildStackFigure()")
		
		act = thisfig.ActorByInternalName("top_of_stack")
		actlist = act.Children()
		for a in actlist:
			if a.ItsFigure():
				childfig = a.ItsFigure()
				figname = childfig.Name()
				if figname[:9] == "BookStack":
					print ("Adding '" + figname +"' to figlist[]")
					self.figlist.append(figname)
					self.getAnyChildStackFigure(childfig)
					
		return
	
	def main(self):
		
		# Get Poser version for doing version-specific stuff
		poserversion = '0'
		posermainversion = '0'
		try:
			self.poserversion = poser.AppVersion()
			temp = self.poserversion.split('.')
			self.posermainversion = temp[0]
		except:
			pass
			
		# Abort if the user gets cold feet right at the start
		confirmed = poser.DialogSimple.YesNo("This script update book/col numbers in the materials.\n\nDo you wish to continue?")
		if not confirmed:
			confirmed = poser.DialogSimple.YesNo("Do you want to see more information about what this script does?\n\n(This opens a debug window with extra information and terminates the script. You'll probably need to resize the debug window to read the information.)")
			if confirmed:
				# Briefly explain to the user what this script does and give an option toabort straight away
				print ("This script randomizes the books in the stack... MORE EXPLANATION TO GO HERE")
			return
			
		# Abort if the selected figure's not a BookStack4
		fig = poser.Scene().CurrentFigure()
		if not (fig.Name()[:9] == "BookStack"):
			poser.DialogSimple.MessageBox("This script is only for use with the 3DCheapskate's 'BookStack' figures.\n\nScript aborted, no changes made.")
			return
		
		# Get parent stacks recursively, and then reverse the list so that the ultimate parent of the whole stack is first
		self.getAnyParentStackFigures(fig)
		self.figlist.reverse()
		
		# ...then add the current figure to the list
		print ("\nAdding '" + fig.Name() +"' to figlist[]")
		self.figlist.append(fig.Name())
		
		# ...then get child stacks recursively
		self.getAnyChildStackFigure(fig)
		
		for l in self.figlist:
			print (l)
		
		# Final confirmation
		confirm1=poser.DialogSimple.YesNo("Ready to upfdate book/col numbers?\n(final confirmation)")
		if confirm1:
			
			bookcount = 0.0
			for f in self.figlist:
				fig = poser.Scene().Figure(f)
				print (fig.Name())
				st = fig.Material("BookMap64Cover").ShaderTree()
				nodelist = st.Nodes()
				for n in nodelist:
					if n.Name() == "COLUMN_0_OR_1":
						col = 0
						if bookcount > 31:
							col = 1
						print ("Set column number to " + str(col))
						n.InputByInternalName("Value_1").SetFloat(col)
					if n.Name() == "BOOK_0_TO_31":
						book = bookcount
						if bookcount > 31:
							book = bookcount - 32
						print ("Set book number to " + str(book))
						n.InputByInternalName("Value_1").SetFloat(book)
						bookcount = bookcount + 4
				
				
			# Redraw what we've modified	
			poser.Scene().Draw()
			
			# Inform the user that it's all done
			poser.DialogSimple.MessageBox("All done")
			
poser.DialogSimple.MessageBox("Running 3DCheapskates SetBookStackMaterial.py script\n\n(N.B. You should see a similar message when the script finishes. If you don't then the script has crashed!)")
cheapskatesTemporaryInstance = cheapskatesTemporaryClass()
cheapskatesTemporaryInstance.main()
poser.DialogSimple.MessageBox("3DCheapskates SetBookStackMaterial.py script all done.")