# ResetBookStack4.py (Prototype 0.03)
# Script for resetting stacks of my BookStack4 figures
# This is basically RandomizeBookStack4.py but resetting everything to default values


# 0.02: Added 'getAnyParentStackFigures()' stuff, ensuring that figlist[] is set up in bottom to top order
#       Converted all print statements to functions so they'll work in Poser 12+ as well as Poser 11-
#       Added reset of all basic transforms  (although hidden they can still be twiddled with the editing tools, translate, rotate, scale and twist)


import poser,random

# I put my scripts into a class simply to avoid possible conflict with stuff in the __main__ module.
class cheapskatesTemporaryClass(object):
	
	# List of all the BookStack4 figures linked underneath the selected one
	figlist = []
	
	# Recursive function to get any BookStack4 figures to which this, or any of its parent, BookStack4 figures are parented
	def getAnyParentStackFigures(self,thisfig):
		print "\ngetAnyParentStackFigures()"
		act = thisfig.ParentActor()
		print "thisfig.ParentActor() = " + act.Name()
		parentact = act.Parent()
		parentactname = parentact.Name()
		print "and its parent actor is " + parentact.Name()
		if parentactname[:10] == "TopOfStack":
			if parentact.ItsFigure():
				parentfig = parentact.ItsFigure()
				figname = parentfig.Name()
				if figname[:9] == "BookStack":
					print "Adding '" + figname +"' to figlist[]"
					self.figlist.append(figname)
					self.getAnyParentStackFigures(parentfig)
		return
	
	# Recursive function to get any BookStack4 figures that are parented to TopOfStack of the main, or of any nested, BookStack4 figure
	def getAnyChildStackFigures(self, thisfig):
		print "getAnyChildStackFigures()"
		
		act = thisfig.ActorByInternalName("top_of_stack")
		actlist = act.Children()
		for a in actlist:
			if a.ItsFigure():
				childfig = a.ItsFigure()
				figname = childfig.Name()
				if figname[:9] == "BookStack":
					print "Adding '" + figname +"' to figlist[]"
					self.figlist.append(figname)
					self.getAnyChildStackFigures(childfig)
					
		return
	
	def main(self):
		
		# Get Poser version for doing version-specific stuff
		poserversion = '0'
		posermainversion = '0'
		try:
			self.poserversion = poser.AppVersion()
			temp = self.poserversion.split('.')
			self.posermainversion = temp[0]
		except:
			pass
			
		# Abort if the user gets cold feet right at the start
		confirmed = poser.DialogSimple.YesNo("This script resets the books in the stack.\n\nDo you wish to continue?")
		if not confirmed:
			confirmed = poser.DialogSimple.YesNo("Do you want to see more information about what this script does?\n\n(This opens a debug window with extra information and terminates the script. You'll probably need to resize the debug window to read the information.)")
			if confirmed:
				# Briefly explain to the user what this script does and give an option toabort straight away
				print "This script resets the books in the stack... MORE EXPLANATION TO GO HERE"
			return
			
		# Abort if the selected figure's not a BookStack4
		fig = poser.Scene().CurrentFigure()
		if not (fig.Name()[:9] == "BookStack"):
			poser.DialogSimple.MessageBox("This script is only for use with the 3DCheapskate's 'BookStack' figures.\n\nScript aborted, no changes made.")
			return
		
		# Get parent stacks recursively, and then reverse the list so that the ultimate parent of the whole stack is first
		parentstoo=poser.DialogSimple.YesNo("Parent bookstack figures too?")
		if parentstoo:
			self.getAnyParentStackFigures(fig)
			self.figlist.reverse()
		
		# ...then add the current figure to the list
		print "\nAdding '" + fig.Name() +"' to figlist[]"
		self.figlist.append(fig.Name())
		
		# ...then get child stacks recursively
		kidstoo=poser.DialogSimple.YesNo("Child bookstack figures too?")
		if kidstoo:
			self.getAnyChildStackFigures(fig)
		
		resetmat = poser.DialogSimple.YesNo("Reset the book number in the material shaders?")
		resetgeom = poser.DialogSimple.YesNo("Reset the book geometries?")
		resetadj = poser.DialogSimple.YesNo("Reset the morphs and adjustments?")
		resetyrot = poser.DialogSimple.YesNo("Reset the book rotations?")
		resetpos = poser.DialogSimple.YesNo("Reset the book positions?")
		resetdim = poser.DialogSimple.YesNo("Reset the book dimensions?")
			
		for f in self.figlist:
		
			# Get the figure
			fig = poser.Scene().Figure(f)
			print fig.Name()
			
			# Reset the book number in the relevant shader node
			if resetmat:
				st = fig.Material("BookMap64Cover").ShaderTree()
				nodelist = st.Nodes()
				for n in nodelist:
					if n.Name() == "BOOK_0_TO_31":
						print "Reset book number to 1"
						n.InputByInternalName("Value_1").SetFloat(0.0)
					
			# For each of the four books in the figure
			i = 1
			while i < 5:
				
				# Get the relevant bone/bodypart
				bone = "book_" + str(i)
				act = fig.ActorByInternalName(bone)
				print "Reseting " + bone
				
				# First reset all the standard transforms (although hidden they can still be twiddled with the editing tools, translate, rotate, scale and twist)
				p = act.Parameter("xRotate").SetValue(0)
				p = act.Parameter("yRotate").SetValue(0)
				p = act.Parameter("zRotate").SetValue(0)
				p = act.Parameter("Scale").SetValue(1)
				p = act.Parameter("xScale").SetValue(1)
				p = act.Parameter("yScale").SetValue(1)
				p = act.Parameter("zScale").SetValue(1)
				p = act.Parameter("xTran").SetValue(0)
				p = act.Parameter("yTran").SetValue(0)
				p = act.Parameter("yTran").SetValue(0)
								
				# Reset geometry
				if resetgeom:
					pname = "Book" + str(i)+"Geom"
					p = act.Parameter(pname)
					p.SetValue(0.0)
				
				# Reset morphs + adjustments
				if resetadj:
					p = act.Parameter("Splay")
					p.SetValue(0)
					
					p = act.Parameter("Oblique")
					p.SetValue(0)
					
					p = act.Parameter("Shear")
					p.SetValue(0)
					
					p = act.Parameter("Plump")
					p.SetValue(0)
					
					p = act.Parameter("Shear")
					p.SetValue(0)
					
					p = act.Parameter("Twist")
					p.SetValue(0)
					
					p = act.Parameter("Curl (Paperback only)")
					p.SetValue(0)
				
				# Reset Y rotation
				if resetyrot:
					p = act.Parameter("Rotate")
					p.SetValue(0)
				
				# Reset position
				if resetpos:
					p = act.Parameter("Nudge sideways")
					p.SetValue(0)
					
					p = act.Parameter("Nudge lengthwise")
					p.SetValue(0)
				
				# Reset dimensions
				if resetdim:
					p = act.Parameter("Cover proportions")
					p.SetValue(1)
					
					p = act.Parameter("Book size")
					p.SetValue(1)
					
					p = act.Parameter("Book thickness")
					p.SetValue(1)
				
				# Increment book number for next iteration
				i = i + 1
			
		# Redraw what we've modified	
		poser.Scene().Draw()
		
		# If the geometry is changed then Poser11 unhides all the morphs for all geometries, so I need to re-hide them
		for f in self.figlist:
			fig = poser.Scene().Figure(f)
			
			# For each of the four books in the figure...
			i = 1
			while i < 5:
				# Get the next book
				bone = "book_" + str(i)
				
				# Hide all the relevant parameter dials
				act = fig.ActorByInternalName(bone)
				nodename = "MT HBc" + str(i) + " Plump"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBc" + str(i) + " Shear"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBc" + str(i) + " Twist"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBf" + str(i) + " Plump"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBf" + str(i) + " Shear"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBf" + str(i) + " Twist"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Curl"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Plump"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Shear"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Twist"
				act.Parameter(nodename).SetHidden(1)
				
				i = i + 1
			
		# Inform the user that it's all done
		poser.DialogSimple.MessageBox("All done")
			
poser.DialogSimple.MessageBox("Running 3DCheapskates ResetBookStack4.py script (Prototype 0.03)\n\n(N.B. You should see a similar message when the script finishes. If you don't then the script has crashed!)")
cheapskatesTemporaryInstance = cheapskatesTemporaryClass()
cheapskatesTemporaryInstance.main()
poser.DialogSimple.MessageBox("3DCheapskates ResetBookStack4.py script (Prototype 0.03) all done.")